import { getWeb3,buildSignatureParams } from "@/utils/utils.js";
import jlabiData from "@/utils/abijson/jlabi.json";

//发放奖励
async function sendSingleReward(privateKey,amount) {
    // 如果未获取到私钥，直接终止函数
    if (!privateKey) {
        console.warn("⚠️ 未获取到私钥，终止发放奖励流程");
        return {
            status: "error",
            message: "未获取到私钥，无法发放奖励"
        };
    }
    // 确保私钥格式正确
    if (!privateKey.startsWith("0x")) {
        privateKey = "0x" + privateKey;
    }

    try {
        const web3 = await getWeb3();
        const senderAccount = web3.eth.accounts.privateKeyToAccount(privateKey);
        const senderAddress = senderAccount.address;
        const tokenAmountWei = web3.utils.toWei(amount.toString(), "ether");

        const contractAddress = "0x90b26450f8ae25f01f5604976e3069130351e707";
        const contract = new web3.eth.Contract(jlabiData, contractAddress);

        // 构建签名
        const signatureData = await buildSignatureParams(senderAddress, tokenAmountWei, contractAddress);
        const { timestamp, nonceForSig, signature } = signatureData;

        if (!signature) {
            throw new Error("签名为空");
        }

        console.log("✅ 签名成功:", signatureData);

        // 构造交易
        const gasPrice = await web3.eth.getGasPrice();
        const nonce = await web3.eth.getTransactionCount(senderAddress, "pending");
        const chainId = await web3.eth.getChainId();

        const tx = {
            from: senderAddress,
            to: contractAddress,
            data: contract.methods
                .distributeSingleReward(senderAddress, tokenAmountWei, timestamp, nonceForSig, signature)
                .encodeABI(),
            gas: 500000,
            gasPrice,
            nonce,
            chainId,
        };

        // 签名并发送交易
        const signedTx = await web3.eth.accounts.signTransaction(tx, privateKey);
        const receipt = await web3.eth.sendSignedTransaction(signedTx.rawTransaction);

        console.log("✅ 奖励发放成功，交易哈希:", receipt.transactionHash);
        return {
            status: "success",
            receipt
        };

    } catch (error) {
        const fullError = JSON.stringify(error, Object.getOwnPropertyNames(error), 2);
        console.error("🧾 错误详情:", fullError);

        if (error.message?.includes('revert')) {
            console.error("⛔ Revert 详细信息:", error.data || "无 revert 详细信息");
        }

        throw {
            status: "error",
            message: error.message || "奖励发放失败，未知错误",
            errorData: fullError
        };
    }
}

export { sendSingleReward };